/*
 * Copyright (c) 2008, Noel Lemouel
 * All rights reserved. See LICENCE file for details.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>

#define LLEN 12	/* # of values per lines */

static int sc_open(FILE**, const char*, struct stat*);
static int so_create(FILE**, const char*);
static void usage(char*);

int bin2c(FILE** fd, unsigned char* buf, char* array_name, unsigned int len)
{
	int i=0;

	fprintf(*fd, "unsigned char %s[%d] = {\n", array_name, len);

	len--;	/* special format for the last value */

	while(len--){
		fprintf(*fd, "0x%02x, ", *(buf++));
		if(i==(LLEN-1)){
			fprintf(*fd, "\n");
			i=0;
		}else{
			i++;
		}
	}
	fprintf(*fd, "0x%02x\n};\n", *buf);

	return 0;
}

int main(int argc, char* argv[])
{
	FILE* fd0;
	FILE* fd1;
	struct stat info;

	unsigned char* sc_buf;

	if(argc < 3){
		usage(argv[0]);
		return -1;
	}else{
		if( sc_open(&fd0, argv[2], &info) ) return -1;
		if( so_create(&fd1, argv[3]) ) return -1;
	}

	sc_buf = malloc(info.st_size);
	if(!sc_buf){
		printf("error: [alloc. %d] failed\n", (int)info.st_size);
		return -1;
	}else{
		printf("[alloc. %d]\n", (int)info.st_size);
	}

	/* ... */

	fread(sc_buf, 1, info.st_size, fd0);
	bin2c(&fd1, sc_buf, argv[1], (unsigned int)info.st_size);

	/* ... */

	free(sc_buf);
	fclose(fd0);
	fclose(fd1);

	return 0;
}

static int sc_open(FILE** fd, const char* name, struct stat* info)
{
      if (stat(name, info) == -1)
      {
          if (errno == ENOENT)
          {
            (void) fprintf( stderr, "error: '%s' does not exist\r\n", name );
          }else{
            (void) fprintf( stderr, "error: stat() fails for '%s'\r\n", name );
          }
          return -1;
      }

      /* checking S_ISLNK(m) is not required since 'stat' follows sym. links */
      if (!S_ISREG(info->st_mode))
      {
         (void) fprintf( stderr, "error: '%s' is a not a regular file (or a sym. link)\r\n", name);
         return -1;
      }

      if ((*fd = fopen(name, "r")) == NULL)
      {
         (void) fprintf( stderr, "error: '%s' open failed\r\n", name );
         return -1;
      }

	  /* printf("size: %d\n", (int)info->st_size); */

      return 0;   /* no error */
}

static int so_create(FILE** fd, const char* name)
{
	struct stat so;
	unsigned char c;

    if (stat(name, &so) == -1)
    {
		if (errno == ENOENT)
        {
            /* does not exist => create it */
			if ((*fd = fopen(name, "w+")) == NULL)
      		{
         		(void) fprintf( stderr, "error: '%s' open failed\r\n", name );
         		return -1;
			}
        }else{
            (void) fprintf( stderr, "error: stat() fails for '%s'\r\n", name );
        	return -1;
        }
    }else if (S_ISREG(so.st_mode)){
		/* file exist => overwrite ? */		
		while(c != 'y' && c != 'n'){
			printf("%s is already existing, overwrite (y/n) ? ", name);

			c = 'y'; /* getchar(); fix this */
			printf("\n");

			/*
				#include <termios.h>
				...
				struct termios t0, t1;
				tcgetattr(0, &t0);
				...
				t1.c_lflag &= ~ECHO; // echo off
				t1.c_lflag &= ~ICANON; // one char at a time
			*/
		}
		if(c == 'y')
		{
      		if ((*fd = fopen(name, "w+")) == NULL)
      		{
         		(void) fprintf( stderr, "error: '%s' open failed\r\n", name );
         		return -1;
			}
		}else{
			return -1;
		}
	}

    return 0;   /* no error */
}

static void usage(char* s)
{
	printf("usage: %s array_name input.bin output.c/.h\n", s);
}
